# Generated by roxygen2: do not edit by hand

export(get_data_series)
export(get_data_series_filter)
export(get_data_table)
export(get_filter_shortcuts)
export(get_metadata_classifications)
export(get_metadata_operation_table)
export(get_metadata_operations)
export(get_metadata_periodicity)
export(get_metadata_publication_dates)
export(get_metadata_publications)
export(get_metadata_series)
export(get_metadata_series_filter)
export(get_metadata_series_operation)
export(get_metadata_series_table)
export(get_metadata_series_values)
export(get_metadata_series_varval)
export(get_metadata_table_groups)
export(get_metadata_table_values)
export(get_metadata_table_varval)
export(get_metadata_tables_operation)
export(get_metadata_values)
export(get_metadata_variables)
